GMGenie.Tickets = {};

-- config
GMGenie.Tickets.perPage = 10;

-- vars
GMGenie.Tickets.pages = 1;
GMGenie.Tickets.tickets = 0;
GMGenie.Tickets.currentPage = 1;
GMGenie.Tickets.currentTicket = { ["num"] = 0, ["ticketId"] = 0, ['name'] = "", ["message"] = "" };
GMGenie.Tickets.order = "ticketId";
GMGenie.Tickets.ascDesc = false;
GMGenie.Tickets.messageOpen = false;
GMGenie.Tickets.done = 0;

-- ticket list
GMGenie.Tickets.list = {};
GMGenie.Tickets.read = {};
GMGenie.Tickets.idToNum = {};

-- refresh ticket list & schedule next refresh
function GMGenie.Tickets.refresh()
    if not GMGenie.Tickets.tempList then
        -- remove scheduled refresh, if any
        Chronos.unscheduleByName('ticketrefresh');
        -- create empty list
        GMGenie.Tickets.tempList = {};
        GMGenie.Tickets.idToNum = {};
        GMGenie.Tickets.tickets = 0;
        -- get ticket list
        if GMGenie_SavedVars.showOfflineTickets then
            SendChatMessage(".ticket list", "GUILD");
        else
            SendChatMessage(".ticket onlinelist", "GUILD");
        end
        
        -- schedule next refresh
        Chronos.scheduleByName('ticketreupdate', 5, GMGenie.Tickets.update);
        Chronos.scheduleByName('ticketrefresh', 60, GMGenie.Tickets.refresh);
    end
end

-- add ticket from chat list to the addon list
function GMGenie.Tickets.listTicket(ticketId, name, createStr, createStamp, lastModifiedStr, lastModifiedStamp)
    local ticketInfo = { ["ticketId"] = ticketId, ["name"] = name, ["createStr"] = createStr, ["createStamp"] = createStamp, ["lastModifiedStr"] = lastModifiedStr, ["lastModifiedStamp"] = lastModifiedStamp, ["assignedTo"] = "" };
    if GMGenie.Tickets.tempList and not GMGenie.Tickets.idToNum[ticketId] then
        -- add to temp list if page is being refreshed
        table.insert(GMGenie.Tickets.tempList, ticketInfo);
        GMGenie.Tickets.tickets = GMGenie.Tickets.tickets + 1;
        GMGenie.Tickets.idToNum[ticketId] = GMGenie.Tickets.tickets;
    elseif not GMGenie.Tickets.idToNum[ticketId] then
        -- else add to current list
        table.insert(GMGenie.Tickets.list, ticketInfo);
        GMGenie.Tickets.tickets = GMGenie.Tickets.tickets + 1;
        GMGenie.Tickets.idToNum[ticketId] = GMGenie.Tickets.tickets;
    else
        GMGenie.Tickets.list[GMGenie.Tickets.idToNum[ticketId]] = ticketInfo;
    end
    -- if no new tickets come in the chat for 1 second, update the list
    Chronos.scheduleByName('ticketreupdate', 0.5, GMGenie.Tickets.update);
end

-- set assignedTo for a ticket
function GMGenie.Tickets.setAssigned(ticketId, assignedTo)
    -- ticket list currently being refreshed or not?
    if GMGenie.Tickets.tempList then
        GMGenie.Tickets.tempList[GMGenie.Tickets.idToNum[ticketId]]["assignedTo"] = assignedTo;
    else
        GMGenie.Tickets.list[GMGenie.Tickets.idToNum[ticketId]]["assignedTo"] = assignedTo;
    end
end

-- update ticket list
function GMGenie.Tickets.update()
    -- move temp list to current list and empty temp list
    if GMGenie.Tickets.tempList then
        GMGenie.Tickets.list = GMGenie.Tickets.tempList;
        GMGenie.Tickets.tempList = nil;
    end
    -- calc number of pages
    GMGenie.Tickets.pages = math.ceil(GMGenie.Tickets.tickets/GMGenie.Tickets.perPage);
    -- allways at least 1 page
    if GMGenie.Tickets.pages < 1 then
        GMGenie.Tickets.pages = 1;
    end
    -- does the page currently being viewed still exist?
    if GMGenie.Tickets.currentPage > GMGenie.Tickets.pages then
        GMGenie.Tickets.currentPage = GMGenie.Tickets.pages;
    end
    -- order ticket list
    GMGenie.Tickets.sort();
end

-- change ordering for ticket list
function GMGenie.Tickets.changeOrder(order)
    if GMGenie.Tickets.order == order then
        if GMGenie.Tickets.ascDesc then
            GMGenie.Tickets.ascDesc = false;
        else
            GMGenie.Tickets.ascDesc = true;
        end
    else
        GMGenie.Tickets.order = order;
        GMGenie.Tickets.ascDesc = false;
    end
    GMGenie.Tickets.currentPage = 1;
    GMGenie.Tickets.sort();
end

-- order ticket list
function GMGenie.Tickets.sort()
    if GMGenie.Tickets.ascDesc then
        table.sort(GMGenie.Tickets.list, function(a,b) return a[GMGenie.Tickets.order]>b[GMGenie.Tickets.order] end);
    else
        table.sort(GMGenie.Tickets.list, function(a,b) return a[GMGenie.Tickets.order]<b[GMGenie.Tickets.order] end);
    end
    
    -- update idToNum table
    GMGenie.Tickets.idToNum = {};
    for ticketNum, ticketInfo in ipairs(GMGenie.Tickets.list) do
        GMGenie.Tickets.idToNum[ticketInfo["ticketId"]] = ticketNum;
    end
    
    -- update the ticket window
    GMGenie.Tickets.updateView();
end

-- update the ticket window
function GMGenie.Tickets.updateView()
    -- Page x of y (z tickets)
    GMGenie_Tickets_Main_Info_Text:SetText("Page ".. GMGenie.Tickets.currentPage .." of ".. GMGenie.Tickets.pages.." (".. GMGenie.Tickets.tickets .." tickets, " ..GMGenie.Tickets.done.. " done)");
    
    -- previous page
    if (GMGenie.Tickets.currentPage == 1) then
		GMGenie_Tickets_Main_Previous:Disable();
	else
		GMGenie_Tickets_Main_Previous:Enable();
	end
    
    -- next page
    if (GMGenie.Tickets.currentPage == GMGenie.Tickets.pages) then
		GMGenie_Tickets_Main_Next:Disable();
	else
		GMGenie_Tickets_Main_Next:Enable();
	end
    
    -- start and end of the list on the current page
    local minTicket = 1 + ((GMGenie.Tickets.currentPage - 1) * GMGenie.Tickets.perPage);
	local maxTicket = GMGenie.Tickets.currentPage * GMGenie.Tickets.perPage;
    local num = 1;
    local i = 0;
    
    -- reset num
    if GMGenie.Tickets.currentTicket["num"] > 0 then
        getglobal("TicketStatusButton".. GMGenie.Tickets.currentTicket["num"] .."_Texture"):SetTexture(0, 0, 0);
    end
    GMGenie.Tickets.currentTicket["num"] = 0;
    
    -- loop through tickets
    for ticketNum, ticketInfo in ipairs(GMGenie.Tickets.list) do
        i = i + 1;
        -- Show ticket?
        if i >= minTicket and i <= maxTicket then
            -- ticket currently open? Set background
            if(ticketInfo["ticketId"] == GMGenie.Tickets.currentTicket["ticketId"]) then
                getglobal("TicketStatusButton".. num .."_Texture"):SetTexture("Interface\\FriendsFrame\\UI-FriendsFrame-HighlightBar");
                GMGenie.Tickets.currentTicket["num"] = num;
            else
                getglobal("TicketStatusButton".. num .."_Texture"):SetTexture(0, 0, 0);
            end
            -- set ticket info
            getglobal("TicketStatusButton".. num .."_ticketId"):SetText(ticketInfo["ticketId"]);
            getglobal("TicketStatusButton".. num .."_name"):SetText(ticketInfo["name"]);
            getglobal("TicketStatusButton".. num .."_createStr"):SetText(ticketInfo["createStr"]);
            getglobal("TicketStatusButton".. num .."_lastModifiedStr"):SetText(ticketInfo["lastModifiedStr"]);
            getglobal("TicketStatusButton".. num .."_assignedTo"):SetText(ticketInfo["assignedTo"]);
            getglobal("TicketStatusButton".. num):Show();
            
            getglobal("TicketStatusButton".. num).ticketId = ticketInfo["ticketId"];
            
            -- ticket is read?
			if GMGenie.Tickets.read[ticketInfo["ticketId"]] then
				getglobal("TicketStatusButton".. num .."_name"):SetTextColor(1,1,1);
			else
				getglobal("TicketStatusButton".. num .."_name"):SetTextColor(0.75,0.75,1);
			end
            -- number on the ticket window
            num = num + 1;
        end
    end
    if num <= GMGenie.Tickets.perPage then
        for num = num, GMGenie.Tickets.perPage do
            getglobal("TicketStatusButton".. num):Hide();
        end
    end
end

-- next page
function GMGenie.Tickets.goToNext()
    if GMGenie.Tickets.currentPage < GMGenie.Tickets.pages then
        GMGenie.Tickets.currentPage = GMGenie.Tickets.currentPage + 1;
        GMGenie.Tickets.updateView();
    end
end

-- previous page
function GMGenie.Tickets.goToPrevious()
    if GMGenie.Tickets.currentPage > 1 then
        GMGenie.Tickets.currentPage = GMGenie.Tickets.currentPage - 1;
        GMGenie.Tickets.updateView();
    end
end

-- mark ticket as read
function GMGenie.Tickets.markAsRead(ticketId)
	GMGenie.Tickets.read[ticketId] = true;
	GMGenie.Tickets.updateView();
end

-- mark ticket as unread
function GMGenie.Tickets.markAsUnread(ticketId)
	GMGenie.Tickets.ReadTickets[ticketId] = false;
end

function GMGenie.Tickets.isOpen()
    local frame = getglobal("GMGenie_Tickets_Main");
	if (frame) then
        return frame:IsVisible();
    end
end

-- hide or show ticket window
function GMGenie.Tickets.toggle()
	if GMGenie.Tickets.isOpen() then
        -- stop auto-refresh
        Chronos.unscheduleByName('ticketrefresh');
        -- hide window
        getglobal("GMGenie_Tickets_Main"):Hide();
    else
        -- refresh ticket list and initiate auto-refresh
        GMGenie.Tickets.refresh();
        -- show window
        getglobal("GMGenie_Tickets_Main"):Show();
    end
end

-- load ticket
function GMGenie.Tickets.loadTicket(ticketId, num)
    if (GMGenie.Tickets.currentTicket["ticketId"] == ticketId) then
		GMGenie.Tickets.close();
    else
        -- un-highlight previous ticket in list
        if GMGenie.Tickets.currentTicket["num"] > 0 then
            getglobal("TicketStatusButton".. GMGenie.Tickets.currentTicket["num"] .."_Texture"):SetTexture(0, 0, 0);
        end
        -- update current ticket
        GMGenie.Tickets.currentTicket = { ["num"] = num, ["ticketId"] = ticketId, ["name"] = GMGenie.Tickets.list[GMGenie.Tickets.idToNum[ticketId]]["name"] };
        -- set title and loading text
        getglobal("GMGenie_Tickets_View_Title_Text"):SetText(GMGenie.Tickets.currentTicket["name"].."'s Ticket");
        GMGenie.Tickets.currentTicket["message"] = "Loading...";
        GMGenie.Tickets.showMessage();
        -- hide reading frame UNUSED ATM
        --getglobal("GMGenie_Tickets_View_Ticket_Reading"):Hide();
        -- get ticket
        SendChatMessage(".ticket viewid ".. ticketId,"GUILD");
        -- open spy
        if GMGenie_SavedVars.useSpy then
            GMGenie.Spy.spy(GMGenie.Tickets.currentTicket["name"]);
        end
        -- mark as read
        GMGenie.Tickets.markAsRead(ticketId);
        -- highlight in list
        getglobal("TicketStatusButton".. num .."_Texture"):SetTexture("Interface\\FriendsFrame\\UI-FriendsFrame-HighlightBar");
        -- toggle frame
		getglobal("GMGenie_Tickets_View"):Show();
        if GMGenie_SavedVars.swapTicketWindows then
            GMGenie.Tickets.toggle();
        end
	end
end

function GMGenie.Tickets.showMessage()
    getglobal("GMGenie_Tickets_View_Ticket_Frame_Text"):SetText(GMGenie.Tickets.currentTicket["message"]);
end

function GMGenie.Tickets.close()
    if GMGenie.Tickets.currentTicket["num"] > 0 then
        getglobal("TicketStatusButton".. GMGenie.Tickets.currentTicket["num"] .."_Texture"):SetTexture(0, 0, 0);
    end
    if GMGenie.Spy.currentRequest["name"] == GMGenie.Tickets.currentTicket["name"] then
        GMGenie_Spy_InfoWindow:Hide();
    end
    getglobal("GMGenie_Tickets_View"):Hide();
    GMGenie.Tickets.currentTicket = { ["num"] = 0, ["ticketId"] = 0, ['name'] = "" };
    if GMGenie_SavedVars.swapTicketWindows then
        GMGenie.Tickets.toggle();
    end
end

-- read ticket
function GMGenie.Tickets.readTicket(ticketId, message)
    if GMGenie.Tickets.currentTicket["ticketId"] == ticketId then
        GMGenie.Tickets.currentTicket["message"] = message;
        GMGenie.Tickets.showMessage();
        return true;
    end
    return false;
end

--add line to ticket
function GMGenie.Tickets.addLine(message)
    GMGenie.Tickets.currentTicket["message"] = GMGenie.Tickets.currentTicket["message"].."\n"..message;
    GMGenie.Tickets.showMessage();
end

function GMGenie.Tickets.delete()
    SendChatMessage(".ticket close "..GMGenie.Tickets.currentTicket["ticketId"], "GUILD");
    SendChatMessage(".ticket del "..GMGenie.Tickets.currentTicket["ticketId"], "GUILD");
    GMGenie.Tickets.done = GMGenie.Tickets.done + 1;
    GMGenie_Tickets_Main_Info_Text:SetText("Page ".. GMGenie.Tickets.currentPage .." of ".. GMGenie.Tickets.pages.." (".. GMGenie.Tickets.tickets .." tickets, " ..GMGenie.Tickets.done.. " done)");
    GMGenie.Tickets.close();
    GMGenie.Tickets.refresh();
end

function GMGenie.Tickets.showInfo(ticketId)
    local ticketInfo = GMGenie.Tickets.list[GMGenie.Tickets.idToNum[ticketId]];
end

function GMGenie.Tickets.assignToSelf()
    SendChatMessage(".ticket assign "..GMGenie.Tickets.currentTicket["ticketId"].." "..UnitName("player"), "GUILD");
end

function GMGenie.Tickets.assign()
    getglobal("GMGenie_Tickets_AssignPopup"):Show();
    getglobal("GMGenie_Tickets_AssignPopup_GMName"):SetText("");
end

function GMGenie.Tickets.assignTo()
    getglobal("GMGenie_Tickets_AssignPopup"):Hide();
    SendChatMessage(".ticket assign "..GMGenie.Tickets.currentTicket["ticketId"].." "..getglobal("GMGenie_Tickets_AssignPopup_GMName"):GetText(), "GUILD");
end

function GMGenie.Tickets.comment()
    getglobal("GMGenie_Tickets_CommentPopup"):Show();
    getglobal("GMGenie_Tickets_CommentPopup_Comment"):SetText("");
end

function GMGenie.Tickets.setComment()
    getglobal("GMGenie_Tickets_CommentPopup"):Hide();
    SendChatMessage("ticket comment "..GMGenie.Tickets.currentTicket["ticketId"].." "..getglobal("GMGenie_Tickets_CommentPopup_Comment"):GetText(), "GUILD");
end

function GMGenie.Tickets.toggleSpy()
    if getglobal("GMGenie_Spy_InfoWindow"):IsVisible() and GMGenie.Tickets.currentTicket["name"] == GMGenie.Spy.currentRequest["name"] then
        getglobal("GMGenie_Spy_InfoWindow"):Hide();
    else
        GMGenie.Spy.spy(GMGenie.Tickets.currentTicket["name"]);
    end
end

-- add slash command to open.close ticket widnow
SLASH_TICKETS1 = "/tickets";
SlashCmdList["TICKETS"] = GMGenie.Tickets.toggle;